/**
 * @file
 * Visitors admin behaviors.
 */

(function visitorAdminForm($) {
  /**
   * Provide the summary information for the tracking settings vertical tabs.
   */
  Drupal.behaviors.trackingSettingsSummary = {
    attach() {
      // Make sure this behavior is processed only if drupalSetSummary is defined.
      if (typeof jQuery.fn.drupalSetSummary === 'undefined') {
        return;
      }

      $('#edit-domain-tracking').drupalSetSummary(
        function domainSummary(context) {
          const $radio = $(
            'input[name="visitors_domain_mode"]:checked',
            context,
          );
          if ($radio.val === '0') {
            return Drupal.t('A single domain');
          }
          if ($radio.val === '1') {
            return Drupal.t('One domain with multiple subdomains');
          }
        },
      );

      $('#edit-page-visibility-settings').drupalSetSummary(
        function pageVisibilitySummary(context) {
          const $radio = $(
            'input[name="visitors_visibility_request_path_mode"]:checked',
            context,
          );
          if ($radio.val === '0') {
            if (
              !$(
                'textarea[name="visitors_visibility_request_path_pages"]',
                context,
              ).val
            ) {
              return Drupal.t('Not restricted');
            }

            return Drupal.t('All pages with exceptions');
          }

          return Drupal.t('Restricted to certain pages');
        },
      );

      $('#edit-role-visibility-settings').drupalSetSummary(
        function roleVisibilitySummary(context) {
          const vals = [];
          $('input[type="checkbox"]:checked', context).each(
            function roleCheckboxVisibility() {
              const label = $(this).next('label');
              vals.push(label.text());
            },
          );
          if (!vals.length) {
            return Drupal.t('Not restricted');
          }
          if (
            $(
              'input[name="visitors_visibility_user_role_mode"]:checked',
              context,
            ).val === '1'
          ) {
            return Drupal.t('Excepted: @roles', { '@roles': vals.join(', ') });
          }

          return vals.join(', ');
        },
      );

      $('#edit-user-visibility-settings').drupalSetSummary(
        function userVisibilitySummary(context) {
          const radio = $(
            'input[name="visitors_visibility_user_account_mode"]:checked',
            context,
          );
          if (radio.val() === '0') {
            return Drupal.t('Not customizable');
          }
          if (radio.val() === '1') {
            return Drupal.t('On by default with opt out');
          }

          return Drupal.t('Off by default with opt in');
        },
      );

      $('#edit-privacy').drupalSetSummary(function privacySummary(context) {
        const vals = [];
        const cookies = $(
          'input#edit-visitors-privacy-disablecookies',
          context,
        );
        if (cookies.is(':checked')) {
          vals.push(Drupal.t('Cookies disabled'));
        }
        if (!vals.length) {
          return Drupal.t('No privacy');
        }
        return Drupal.t('@items', { '@items': vals.join(', ') });
      });

      $('#edit-status-codes').drupalSetSummary(function statusSummary(context) {
        const vals = [];
        const code404 = $('input#edit-status-codes-disabled-404', context);
        if (code404.is(':checked')) {
          vals.push(Drupal.t('404'));
        }
        const code403 = $('input#edit-status-codes-disabled-403', context);
        if (code403.is(':checked')) {
          vals.push(Drupal.t('403'));
        }
        if (!vals.length) {
          return Drupal.t('Default');
        }
        return Drupal.t('@items disabled', { '@items': vals.join(', ') });
      });

      $('#edit-entity').drupalSetSummary(function entitySummary(context) {
        const vals = [];
        const counter = $('input#edit-counter-enabled', context);
        if (!counter.is(':checked')) {
          vals.push(Drupal.t('Disabled'));
        }
        if (!vals.length) {
          return Drupal.t('Default');
        }
        return Drupal.t('@items', { '@items': vals.join(', ') });
      });

      $('#edit-performance-track').drupalSetSummary(
        function entitySummary(context) {
          const vals = [];
          const performance = $('input#edit-performance', context);
          if (performance.is(':checked')) {
            vals.push(Drupal.t('Tracking'));
          } else {
            vals.push(Drupal.t('Not tracking'));
          }
          const scriptType = $('input[name="script_type"]:checked', context);
          if (scriptType.val() === 'full') {
            vals.push(Drupal.t('Full'));
          } else {
            vals.push(Drupal.t('Minified'));
          }

          return Drupal.t('@items', { '@items': vals.join(', ') });
        },
      );
    },
  };
})(jQuery);
