<?php

namespace Drupal\visitors\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\visitors\VisitorsTrackerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\Response;

/**
 * Visitors tracking controller.
 */
final class Visitors extends ControllerBase {


  /**
   * The tracker service.
   *
   * @var \Drupal\visitors\VisitorsTrackerInterface
   */
  protected $tracker;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $stack;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container): Visitors {
    return new self(
      $container->get('visitors.tracker'),
      $container->get('request_stack')
    );
  }

  /**
   * Visitor tracker.
   *
   * @param \Drupal\visitors\VisitorsTrackerInterface $tracker
   *   The date service.
   * @param \Symfony\Component\HttpFoundation\RequestStack $stack
   *   The request stack.
   */
  public function __construct(VisitorsTrackerInterface $tracker, RequestStack $stack) {
    $this->tracker = $tracker;
    $this->stack = $stack;
  }

  /**
   * Tracks visits.
   */
  public function track(): Response {
    $request = $this->stack->getCurrentRequest();

    $query = $request->query->all();
    $languages = $request->getLanguages() ?? [];

    $this->tracker->log($query, $languages);

    $response = new Response();
    $response->setContent('');

    return $response;
  }

}
