<?php

namespace Drupal\visitors\Plugin\views\field;

use Drupal\Component\Utility\Xss as UtilityXss;
use Drupal\views\Plugin\views\field\FieldPluginBase;
use Drupal\views\Render\ViewsRenderPipelineMarkup;

/**
 * Field handler to display the hour (server) of the visit.
 *
 * @ingroup views_field_handlers
 *
 * @ViewsField("visitors_week")
 */
final class VisitorsWeek extends FieldPluginBase {

  /**
   * {@inheritdoc}
   */
  public function query() {
    $this->ensureMyTable();
    // Add the field.
    $params = $this->options['group_type'] != 'group' ? ['function' => $this->options['group_type']] : [];
    /** @var \Drupal\views\Plugin\views\query\Sql $query */
    $query = $this->query;
    $this->field_alias = $query->addField(NULL, visitors_date_format_sql('visitors_date_time', '%X%V'), 'visitors_week', $params);

    $this->addAdditionalFields();
  }

  /**
   * {@inheritdoc}
   */
  public function render($values) {

    $value = $this->getValue($values);
    $year = (int) substr($value, 0, 4);
    $week = (int) substr($value, 4, 2);

    // Converts week of year to date.
    $date = new \DateTime();
    $date->setISODate($year, $week);

    return ViewsRenderPipelineMarkup::create(UtilityXss::filterAdmin($date->format('Y-m-d')));
  }

}
