<?php

namespace Drupal\visitors;

use DeviceDetector\DeviceDetector;

/**
 * Detects the device type.
 *
 * @package Drupal\visitors
 */
interface VisitorsDeviceInterface {

  /**
   * Bulk updates missing device information.
   *
   * @param string $user_agent
   *   The user agent string.
   *
   * @return int
   *   The number of records updated.
   */
  public function bulkUpdate(string $user_agent): int;

  /**
   * Gets the unique user agents.
   *
   * @return array
   *   The unique user agents.
   */
  public function getUniqueUserAgents(): array;

  /**
   * Gets the device detector.
   *
   * @param string $user_agent
   *   The user agent string.
   * @param array $server
   *   The server array.
   *
   * @return \DeviceDetector\DeviceDetector
   *   The device detector.
   */
  public function getDeviceDetector(string $user_agent, ?array $server = NULL): DeviceDetector;

}
