<?php

declare(strict_types=1);

namespace Drupal\Tests\visitors\Unit\Form;

use Drupal\Tests\UnitTestCase;
use Drupal\visitors\Form\Referer;
use Symfony\Component\DependencyInjection\ContainerBuilder;

/**
 * Referer form test.
 *
 * @group visitors
 */
class RefererTest extends UnitTestCase {

  /**
   * The form.
   *
   * @var \Drupal\visitors\Form\Referer
   */
  protected $form;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface|\PHPUnit\Framework\MockObject\MockObject
   */
  protected $languageManager;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $container = new ContainerBuilder();

    $string_translation = $this->getStringTranslationStub();
    $container->set('string_translation', $string_translation);

    $date_formatter = $this->createMock('Drupal\Core\Datetime\DateFormatterInterface');
    $container->set('date.formatter', $date_formatter);

    $database = $this->createMock('Drupal\Core\Database\Connection');
    $container->set('database', $database);

    $this->languageManager = $this->createMock('Drupal\Core\Language\LanguageManagerInterface');
    $container->set('language_manager', $this->languageManager);

    \Drupal::setContainer($container);

    $this->form = Referer::create($container);
  }

  /**
   * Test the form id.
   */
  public function testGetFormId() {
    $this->assertEquals('visitors_referer_form', $this->form->getFormId());
  }

  /**
   * Test the form.
   */
  public function testBuildForm() {

    $current_language = $this->createMock('Drupal\Core\Language\LanguageInterface');
    $current_language->expects($this->any())
      ->method('getId')
      ->willReturn('en');
    $this->languageManager->expects($this->any())
      ->method('getCurrentLanguage')
      ->willReturn($current_language);

    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form = $this->form->buildForm([], $form_state);

    $this->assertIsArray($form);
    $this->assertCount(3, $form);
    $this->assertArrayHasKey('visitors_date_filter', $form);
    $this->assertArrayHasKey('visitors_referer', $form);
    $this->assertArrayHasKey('submit', $form);
  }

  /**
   * Tests submitForm.
   */
  public function testSubmitForm() {
    $date_to = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_to->expects($this->exactly(3))
      ->method('format')
      ->willReturn('2019-01-01');
    $date_from = $this->createMock('Drupal\Core\Datetime\DrupalDateTime');
    $date_from->expects($this->exactly(3))
      ->method('format')
      ->willReturn('2019-01-01');
    $form = [];
    $form_state = $this->createMock('Drupal\Core\Form\FormStateInterface');
    $form_state->expects($this->once())
      ->method('getValues')
      ->willReturn([
        'referer_type' => 'internal',
      ]);
    $form_state->expects($this->exactly(2))
      ->method('getValue')
      ->willReturn($date_to, $date_from);

    $this->form->submitForm($form, $form_state);
  }

}
