<?php

/**
 * @file
 * Provide views data for visitors.module.
 */

use Drupal\Core\Entity\ContentEntityInterface;

/**
 * Implements hook_views_data().
 */
function visitors_views_data() {
  $data = [];
  $data['visitors_counter']['table']['group'] = t('Visitor counters');
  $data['visitors_counter']['table']['base'] = [
    'title' => t('Visitor Counters'),
    'help' => t('Visitors data from visitors DB table.'),
  ];
  $settings = \Drupal::config('visitors.config');
  $supported_entity_types = $settings->get('counter.entity_types') ?? [];
  foreach (\Drupal::entityTypeManager()->getDefinitions() as $entity_type_id => $entity_type) {
    if (!in_array($entity_type_id, $supported_entity_types) || !$entity_type->entityClassImplements(ContentEntityInterface::class) || !$entity_type->getBaseTable()) {
      continue;
    }

    $base_table = $entity_type->getDataTable() ?: $entity_type->getBaseTable();
    $args = ['@entity_type' => $entity_type_id];

    // Multilingual properties are stored in data table.
    if (!($table = $entity_type->getDataTable())) {
      $table = $entity_type->getBaseTable();
    }
    $data[$base_table]['visitors_counter'] = [
      'title' => t('Visitors @entity_type counter', $args),
      'help' => t('Relate all visitor counts on the @entity_type.', $args),
      'relationship' => [
        'group' => t('Visitor Counters'),
        'label' => t('Visitor counters'),
        'base' => 'visitors_counter',
        'base field' => 'entity_id',
        'relationship field' => $entity_type->getKey('id'),
        'id' => 'standard',
        'extra' => [
          [
            'field' => 'entity_type',
            'value' => $entity_type_id,
          ],
        ],
      ],
    ];

    $data['visitors_counter']['table']['join'][$table] = [
      'type' => 'LEFT',
      'left_field' => $entity_type->getKey('id'),
      'field' => 'entity_id',
      'extra' => [
        [
          'field' => 'entity_type',
          'value' => $entity_type_id,
        ],
      ],
    ];

  }

  $data['visitors_counter']['total'] = [
    'title' => t('Total views'),
    'help' => t('The total number of times the node has been viewed.'),
    'field' => [
      'id' => 'visitors_numeric',
      'click sortable' => TRUE,
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
  ];
  $data['visitors_counter']['today'] = [
    'title' => t('Views today'),
    'help' => t('The total number of times the node has been viewed today.'),
    'field' => [
      'id' => 'visitors_numeric',
      'click sortable' => TRUE,
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
  ];
  $data['visitors_counter']['timestamp'] = [
    'title' => t('Most recent visit'),
    'help' => t('The most recent time the node has been viewed.'),
    'field' => [
      'id' => 'visitors_counter_timestamp',
      'click sortable' => TRUE,
    ],
    'filter' => [
      'id' => 'date',
    ],
    'argument' => [
      'id' => 'date',
    ],
    'sort' => [
      'id' => 'standard',
    ],
  ];

  $data['visitors']['table']['group'] = t('Visitors');
  $data['visitors']['table']['base'] = [
    'title' => t('Visitors'),
    'help' => t('Visitors data from visitors DB table.'),
  ];

  $data['visitors']['visitors_id'] = [
    'title' => t('Visitors ID'),
    'help' => t('Visitors entry ID.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['visitor_id'] = [
    'title' => t('Unique visitor'),
    'help' => t('A unique ID for the visitor.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_uid'] = [
    'title' => t('Visitors UID'),
    'help' => t('The user ID of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'relationship' => [
      'title' => t('User'),
      'help' => t('The user entity from the visitor entry.'),
      'base' => 'users_field_data',
      'base field' => 'uid',
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['visitors_date_time'] = [
    'title' => t('Visitors Date Time'),
    'help' => t('The timestamp from the visitors entry.'),
    'field' => [
      'id' => 'date',
      'click sortable' => TRUE,
    ],
    'filter' => [
      'id' => 'visitors_date',
    ],
  ];
  $data['visitors']['visitors_hour'] = [
    'title' => t('Hour'),
    'help' => t('The hour (server) of the visit.'),
    'field' => [
      'id' => 'visitors_hour',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitors_month'] = [
    'title' => t('Month'),
    'help' => t('The month of the visit.'),
    'field' => [
      'id' => 'visitors_month',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitors_day_of_week'] = [
    'title' => t('Day of Week'),
    'help' => t('The day of week of the visit.'),
    'field' => [
      'id' => 'visitors_day_of_week',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitors_day_of_month'] = [
    'title' => t('Day of Month'),
    'help' => t('The day of month of the visit.'),
    'field' => [
      'id' => 'visitors_day_of_month',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitors_day'] = [
    'title' => t('Day'),
    'help' => t('The day of the visit.'),
    'field' => [
      'id' => 'visitors_day',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitors_week'] = [
    'title' => t('Week'),
    'help' => t('The week of the visit.'),
    'field' => [
      'id' => 'visitors_week',
      'field' => 'visitors_date_time',
    ],
  ];
  $data['visitors']['visitor_localtime'] = [
    'title' => t('Visitor Hour'),
    'help' => t('The hour (client) of the visit.'),
    'field' => [
      'id' => 'visitors_local_hour',
      'field' => 'visitor_localtime',
    ],
  ];
  $data['visitors']['visitors_ip'] = [
    'title' => t('Visitors IP'),
    'help' => t('The IP of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_url'] = [
    'title' => t('Visitors URL'),
    'help' => t('The URL of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_referer'] = [
    'title' => t('Visitors referer'),
    'help' => t('The referer of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_path'] = [
    'title' => t('Visitors path'),
    'help' => t('The path of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['route'] = [
    'title' => t('Route'),
    'help' => t('The route of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_title'] = [
    'title' => t('Visitors title'),
    'help' => t('The title of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['visitors_user_agent'] = [
    'title' => t('Visitors user agent'),
    'help' => t('The user agent of the visitors entry.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_resolution'] = [
    'title' => t('Resolution'),
    'help' => t("The visitor's screen resolution."),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_pdf'] = [
    'title' => t('PDF Support'),
    'help' => t("The visitor's browser supports PDFs."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_flash'] = [
    'title' => t('Flash Support'),
    'help' => t("The visitor's browser supports Flash."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_java'] = [
    'title' => t('Java Support'),
    'help' => t("The visitor's browser supports Java."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_quicktime'] = [
    'title' => t('Quicktime Support'),
    'help' => t("The visitor's browser supports Quicktime."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_realplayer'] = [
    'title' => t('Realplayer Support'),
    'help' => t("The visitor's browser supports Realplayer."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_windowsmedia'] = [
    'title' => t('Windows Media Support'),
    'help' => t("The visitor's browser supports Windows Media."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_silverlight'] = [
    'title' => t('Silverlight Support'),
    'help' => t("The visitor's browser supports Silverlight."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_cookie'] = [
    'title' => t('Cookie Support'),
    'help' => t("The visitor's browser supports cookies."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['config_browser_engine'] = [
    'title' => t('Browser Engine'),
    'help' => t('The engine used by the browser.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_browser_name'] = [
    'title' => t('Browser Name'),
    'help' => t('The name of the browser.'),
    'field' => [
      'id' => 'visitors_browser',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_browser_version'] = [
    'title' => t('Browser Version'),
    'help' => t('The version of the browser.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_client_type'] = [
    'title' => t('Client type'),
    'help' => t('The type of the client.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_device_brand'] = [
    'title' => t('Device brand'),
    'help' => t('The brand of the device.'),
    'field' => [
      'id' => 'visitors_brand',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_device_model'] = [
    'title' => t('Device model'),
    'help' => t('The model of the device.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_device_type'] = [
    'title' => t('Device type'),
    'help' => t('The type of device.'),
    'field' => [
      'id' => 'visitors_device',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_os'] = [
    'title' => t('Operating System'),
    'help' => t('The operating system.'),
    'field' => [
      'id' => 'visitors_operating_system',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['config_os_version'] = [
    'title' => t('OS version'),
    'help' => t('The version of the Operating System.'),
    'field' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['bot'] = [
    'title' => t('Bot'),
    'help' => t("The visit is from a bot."),
    'field' => [
      'id' => 'boolean',
    ],
    'filter' => [
      'id' => 'boolean',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors']['language'] = [
    'title' => t('Language'),
    'help' => t('The browser language.'),
    'field' => [
      'id' => 'visitors_language',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['location_continent'] = [
    'title' => t('Continent'),
    'help' => t('The location continent.'),
    'field' => [
      'id' => 'visitors_continent',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];
  $data['visitors']['location_country'] = [
    'title' => t('Country'),
    'help' => t('The location country.'),
    'field' => [
      'id' => 'visitors_country',
    ],
    'filter' => [
      'id' => 'string',
    ],
    'argument' => [
      'id' => 'string',
    ],
  ];

  $data['visitors']['visitors_performance'] = [
    'title' => t('Visitors performance'),
    'help' => t('Relate all visitor counts on the.'),
    'relationship' => [
      'group' => t('Visitor performance'),
      'label' => t('Visitor performance'),
      'base' => 'visitors_performance',
      'base field' => 'visitors_id',
      'relationship field' => 'visitors_id',
      'id' => 'standard',
    ],
  ];

  $data['visitors_performance']['table']['group'] = t('Visitor performance');
  $data['visitors_performance']['table']['base'] = [
    'title' => t('Visitor performance'),
    'help' => t('Visitors performance table.'),
  ];

  $data['visitors_performance']['network'] = [
    'title' => t('Network'),
    'help' => t('Network performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['server'] = [
    'title' => t('Server'),
    'help' => t('Server performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['transfer'] = [
    'title' => t('Transfer'),
    'help' => t('Transfer performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['dom_processing'] = [
    'title' => t('DOM Processing'),
    'help' => t('DOM Processing performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['dom_complete'] = [
    'title' => t('DOM Complete'),
    'help' => t('DOM Complete performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['on_load'] = [
    'title' => t('On Load'),
    'help' => t('On Load performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];
  $data['visitors_performance']['total'] = [
    'title' => t('Total'),
    'help' => t('Total performance.'),
    'field' => [
      'id' => 'numeric',
    ],
    'sort' => [
      'id' => 'standard',
    ],
    'filter' => [
      'id' => 'numeric',
    ],
    'argument' => [
      'id' => 'numeric',
    ],
  ];

  $data['visitors']['visitors_display_link'] = [
    'title' => t('Link to Visitors display'),
    'help' => t('Displays a link to a non-path-based display of this view while keeping the filter criteria, sort criteria, pager settings and contextual filters.'),
    'area' => [
      'id' => 'visitors_display_link',
    ],
  ];

  return $data;
}
